@@ -1,5 +1,3 @@
-from numba import njit
-import pygame as pg
 from Retro3D import *
 
 
@@ -10,41 +8,10 @@ from Retro3D import *
 # Author:  Deepak (I can't belive I'm programming Python) Deo
 # Date:    Jan 3, 2023
 #
-# future updates:
-#
-#   display lists should hold tris, not objs
-#       for better optimization + variety of rendering types within
-#       a single object (i.e. solid + transparent, etc)
-#
-#   need proper clipping. should be adding vertices to show part of the poly.
-#
-#   use .mtl file along with .obj so that the obj files can have color per face info
-#   also, need to use 'materials' so that we can color objs on top of what the .obj file specifies.
-#   right now it's just one color per object
-#
-#   support multiple lights, multiple light types (i.e. point), and light color
-#
-#   optimize:
-#       use njit properly
-#           https://numba.readthedocs.io/en/stable/user/performance-tips.html
-#       __draw_normals method
-#
-#       Matrix class should have pre-multipled matrix methods: ie zxyt
-#
-#   make code more 'pythonic'
-#
-#   need simpler/cleaner way of doing imports 
-#
-#   support mouse input for Game
-#
-#   shaded outline shouldn't show the diagonal line of the tri
-#       maybe precalc which line is the longest and flag it
-#
-#
 ###############################################################################
 class Engine:
 
-    VERSION = 1.6
+    VERSION = 2.6
 
     DISPLAY_LIST_WIREFRAME          = 1
     DISPLAY_LIST_SHADED             = 2
@@ -52,14 +19,20 @@ class Engine:
 
 
 
+    #dsd there's probably a pythonic non-looping way to do this
     ###############################################################################
     #
     ###############################################################################
     @staticmethod
     @njit(fastmath=True)
-    def AreAllVertsVisible(list_verts: list, screen_res_half_x: float, screen_res_half_y: float):
-        # NOTE: verts that are offscreen have had their vals pushed to half_x or half_y
-        return not np.any((list_verts == screen_res_half_x) | (list_verts == screen_res_half_y))
+    def AreAllVertsVisible(list_verts: list, screen_res_x: float, screen_res_y: float):
+        for v in list_verts:
+            if v[0] <= 0 or v[0] >= screen_res_x:
+                return False;
+            if v[1] <= 0 or v[1] >= screen_res_y:
+                return False;
+
+        return True;
 
 
     ###############################################################################
@@ -67,27 +40,35 @@ class Engine:
     ###############################################################################
     def __init__(self, config: ConfigGame):
 
-        SiLog.Message("starting", "Retro3D")
-
-        SiLog.Message("version", Engine.VERSION)
+        SiLog.KeyVal("starting Retro3D", Engine.VERSION)
 
         # using pygame (pg) for drawing to the screen
         pg.init()
+    
+        # verify sound will work
+        # pg.mixer.music.load crashes if not devices deteced. wonderful :/
+        self.is_sound_active = True
+        try:
+            pg.mixer.init()
+        except:
+            SiLog.Message("no sound device detected") 
+            self.is_sound_active = False
      
         self.config = config  
 
-        # NOTE: '//' means floor division 
-        #   i.e. 5/2 = 2.5
-        #        5//2 = 2
-        self.screen_res_half = SiVector2(self.config.screen_resolution.x // 2, self.config.screen_resolution.y // 2)
+        self.screen_res = pg.math.Vector2(self.config.screen_resolution.x, self.config.screen_resolution.y)
         self.screen = pg.display.set_mode([self.config.screen_resolution.x, self.config.screen_resolution.y])
 
+        self.screen_rect = pg.Rect(0, 0, self.config.screen_resolution.x, self.config.screen_resolution.y)
+
         self.light = light = LightDirectional(config.light_direction)
 
         self.FPS = 60
         self.clock = pg.time.Clock()
+        self.frame_rate_last_frame_ticks = 0
+        self.delta_time = 0.0
 
-        self.camera = Camera(SiVector3(0, 0, 0))
+        self.camera = Camera(pg.math.Vector3(0, 0, 0))
         self.projection = Projection(self, self.config.screen_resolution)
 
         # must add objects to one of these lists if you want them drawn
@@ -95,6 +76,16 @@ class Engine:
         self.display_list_shaded = list()
         self.display_list_shaded_outline = list()
 
+        
+        base_path_rez = os.path.dirname(__file__) + os.sep + "rez" + os.sep 
+
+        # load engine fonts
+        base_path_font = base_path_rez + "fonts" + os.sep 
+        font_name = base_path_font + "EternalLogo.ttf"
+
+        self.font_engine_large = pg.font.Font(font_name, 100)
+        self.font_engine_medium = pg.font.Font(font_name, 45)
+        self.font_engine_small = pg.font.Font(font_name, 25)
 
     ###############################################################################
     #
@@ -103,35 +94,28 @@ class Engine:
 
         if display_list_type == Engine.DISPLAY_LIST_WIREFRAME:
             self.display_list_wireframe.append(obj)
+            obj.set_display_list(self.display_list_wireframe)
         elif display_list_type == Engine.DISPLAY_LIST_SHADED:
             self.display_list_shaded.append(obj)
+            obj.set_display_list(self.display_list_shaded)
         elif display_list_type == Engine.DISPLAY_LIST_SHADED_OUTLINE:
             self.display_list_shaded_outline.append(obj)
+            obj.set_display_list(self.display_list_shaded_outline)
         else:
-            SiLog.Error("unsupported display list type [" + display_list_type + "]");
+            SiLog.Unsupported(display_list_type);
     
  
-            
     ###############################################################################
     #
     ###############################################################################
-    def draw_gradient(self, color_top: pg.Color, color_bottom: pg.Color, target_rect: pg.Rect, gradient_direction: 'SiDirection'):
-    
-        # draw colors to a tiny rectangle
-        tiny_rect = pg.Surface((2,2)) 
+    def remove_display_object(self, obj: Object):
 
-        if gradient_direction == SiDirection.VERTICAL:
-            pg.draw.line(tiny_rect, color_top, (0,0), (1,0))  
-            pg.draw.line(tiny_rect, color_bottom, (0,1), (1,1))          
-        else:
-            pg.draw.line(tiny_rect, color_top, (0,0), (0,1))  
-            pg.draw.line(tiny_rect, color_bottom, (1,0), (1,1))          
-
-        # scale rectangle up and use smoothing
-        tiny_rect = pg.transform.smoothscale(tiny_rect, (target_rect.width, target_rect.height))
-
-        self.screen.blit(tiny_rect, target_rect) 
+        if obj.get_display_list() == None:
+            SiLog.Error("missing display list!")
+            return
 
+        obj.get_display_list().remove(obj)
+          
 
     ###############################################################################
     #
@@ -142,22 +126,16 @@ class Engine:
         self.screen.fill(pg.Color(0, 0, 0))
 
 
-        for section_info in self.config.background_info.list_section_info:
-        
-            if isinstance(section_info, BackgroundInfo.SectionInfo):
-                pg.draw.rect(self.screen, section_info.color, section_info.rect)
-            elif isinstance(section_info, BackgroundInfo.SectionInfoGradient):
-                self.draw_gradient(section_info.color_start, section_info.color_end, section_info.rect, section_info.direction)
-            else:
-                SiLog.Error("unsupported section info type", section_info)
-
-
-
     ###############################################################################
     #
     ###############################################################################
     def update(self):
-    
+
+        # update delta_time    
+        t = pg.time.get_ticks()
+        self.delta_time = (t - self.frame_rate_last_frame_ticks) / 1000.0
+        self.frame_rate_last_frame_ticks = t
+
         # precalc camera matrix
         camera_matrix = self.camera.calc_camera_matrix()
 
@@ -175,6 +153,10 @@ class Engine:
     ###############################################################################
     def __display(self, camera_matrix: Matrix, display_list: list, display_list_type: int):
 
+        camera_forward = [self.camera.forward.x, self.camera.forward.y, self.camera.forward.z, 0]
+        camera_pos = [self.camera.pos.x, self.camera.pos.y, self.camera.pos.z, 1]
+
+
         for obj in display_list:
 
             # move verts to world space
@@ -205,7 +187,7 @@ class Engine:
             #
             #           in a standard mult - you'd think of the vertex as a vertical col.
             #           but here it's a row - which means we need to mult by the cols of world_mat
-            list_vertex = np.matmul(obj.mesh.list_vertex, obj.mat_world)
+            list_vertex = np.matmul(obj.model.list_vertex, obj.mat_world)
 
             # copy the list to keep original world vertices
             list_vertex_world = list_vertex[:]
@@ -214,7 +196,12 @@ class Engine:
             # NOTE: need normals even if not drawing normals since code
             #       is using the normal to cull any polygons that are not
             #       facing the camera. also needed for lighting
-            list_normal = np.matmul(obj.mesh.list_face_normal, obj.mat_world)
+            # NOTE: removing position info from matrix for normal calcs - akin to moving to camera space... :/
+            mat = obj.mat_world.copy()
+            mat[3][0] = 0.0
+            mat[3][1] = 0.0
+            mat[3][2] = 0.0
+            list_normal = np.matmul(obj.model.list_face_normal, mat)
 
             # move verts to camera space   
             list_vertex = np.matmul(list_vertex, camera_matrix)
@@ -232,33 +219,14 @@ class Engine:
             #           v[3] /= v[3];       # w / w
             list_vertex /= list_vertex[:, -1].reshape(-1, 1)
 
-            # clip verts that are outside of unit cube
-            #   the reason we set the vert to 0 is for the next matrix mult
-            #   i.e. 0 for x means the matrix mult will not get any of the x component 
-            #        but it will get the full y component (which is ie 800).
-            #        'pushing' all those values to the borders which makes it 
-            #        easier for us to check if any of the 2d polygon verts are outside of the screen
-            #        so we won't need to draw them
-            #   this below is just doing this:
-            #
-            #   for i, v in enumerate(list_vertex):
-            #       if v[0] > 1.0:
-            #        list_vertex[i][0] = 0.0
-            #       elif v[1] > 1.0:
-            #           list_vertex[i][1] = 0.0
-            #       elif v[2] > 1.0:
-            #           list_vertex[i][2] = 0.0
-            #       elif v[0] < -1.0:
-            #           list_vertex[i][0] = 0.0
-            #       elif v[1] < -1.0:
-            #           list_vertex[i][1] = 0.0
-            #       elif v[2] < -1.0:
-            #           list_vertex[i][2] = 0.0
-            list_vertex[(list_vertex > 1) | (list_vertex < -1)] = 0
+            # dsd this is where we could clip verts that are outside of the unit cube...
 
             # unit_cube -> screen
             list_vertex = np.matmul(list_vertex, self.projection.to_screen_matrix)
 
+            # store this before moving down to 2d
+            list_vertex_screen = list_vertex[:]
+
             # create a new list with just the x/y of the verts
             # since, in the end, all we are doing is drawing a 2d image...
             list_vertex = list_vertex[:, :2]
@@ -266,6 +234,15 @@ class Engine:
             #draw faces
             for face_idx, face_info in enumerate(obj.list_face_info):
                              
+                # skip drawing if any of the 3 verts are behind the camera
+                is_out_of_view = False
+                for v in list_vertex_screen:
+                    if v[2] == 0.0:
+                        is_out_of_view = True
+                        break
+                if is_out_of_view == True:
+                    continue
+
                 # this below code is just doing this
                 # color = face_info[0]
                 # face = face_info[1]
@@ -282,57 +259,60 @@ class Engine:
                 # any of the 3 vertices will do
                 vertex_world = list_vertex_world[list_vertex_index][0]
 
-                if Engine.AreAllVertsVisible(list_polygon_vertex, self.screen_res_half.x, self.screen_res_half.y):
 
-                    # only draw the face if its normal is facing the camera!                  
+                if Engine.AreAllVertsVisible(list_polygon_vertex, self.screen_res.x, self.screen_res.y):
 
-                    # NOTE: normals are already normalized since we are just rotating a normalized vector
-                    v1 = list_normal[face_idx]
+                    # only draw the face if it's in the view of the camera
+                    # using dot product for this
 
-                    # get vector from face to the camera
-                    v2 = vertex_world - [self.camera.pos.x, self.camera.pos.y, self.camera.pos.z, 1]
+                    # v1 = cam to face
+                    v1 = camera_pos - vertex_world 
+                    v1 = v1/np.linalg.norm(v1)
+                    dp = np.dot(v1, camera_forward)
 
-                    #normalize
-                    v2 = v2/np.linalg.norm(v2)
-
-                    dp = np.dot(v1,v2)
                     if dp < 0.0:
 
-                        # now that we know tri is being drawn, calc dp for lighting
-                        dp = v1[0]*self.light.direction[0] + v1[1]*self.light.direction[1] + v1[2]*self.light.direction[2]
-                        dp = (1.0 - dp) * 0.5
-                        face_color = self.__calc_face_color(color, dp)
+                        #only draw if poly is facing the camera
 
-                        # convert vertices from 4 components [x y z w] to 2 components [x y]   
-                        v0 = [list_polygon_vertex[0][0], list_polygon_vertex[0][1]]
-                        v1 = [list_polygon_vertex[1][0], list_polygon_vertex[1][1]]
-                        v2 = [list_polygon_vertex[2][0], list_polygon_vertex[2][1]]
+                        # NOTE: normals are already normalized since we are just rotating a normalized vector
+                        v1 = list_normal[face_idx]
 
-                        # NOTE: polygon draw with 0 means fill 0 otherwise it means thickness of line (int)
-                        if display_list_type == Engine.DISPLAY_LIST_WIREFRAME:
-                            pg.draw.line(self.screen, color, v0, v1, 2)
-                            pg.draw.line(self.screen, color, v1, v2, 2)
-                            pg.draw.line(self.screen, color, v2, v0, 2)
-                        elif display_list_type == Engine.DISPLAY_LIST_SHADED:
-                            pg.draw.polygon(self.screen, face_color, list_polygon_vertex, 0)  
-                        elif display_list_type == Engine.DISPLAY_LIST_SHADED_OUTLINE:
-                            pg.draw.polygon(self.screen, face_color, list_polygon_vertex, 0)  
-                            pg.draw.line(self.screen, pg.Color('white'), v0, v1, 1)
-                            pg.draw.line(self.screen, pg.Color('white'), v1, v2, 1)
-                            pg.draw.line(self.screen, pg.Color('white'), v2, v0, 1)
-                        else:
-                            SiLog.Error("unsupported display list type [" + display_list_type + "]");
+                        # get vector from face to the camera
+                        v2 = vertex_world - camera_pos
+                        v2 = v2/np.linalg.norm(v2)
 
+                        dp = np.dot(v1,v2)
 
+                        if dp < 0.0:
 
-                    if obj.draw_normals:
-                        self.__draw_normals(list_vertex_world, list_vertex_index, list_normal, face_idx)
- 
-                    if obj.draw_vertices:
-                        for vertex in list_vertex:
-                            pg.draw.circle(self.screen, pg.Color('white'), vertex, 6)
+                            # now that we know tri is being drawn, calc dp for lighting
+                            dp = v1[0]*self.light.direction[0] + v1[1]*self.light.direction[1] + v1[2]*self.light.direction[2]
+                            dp = (1.0 - dp) * 0.5
+                            face_color = self.__calc_face_color(color, dp)
+
+                            # convert vertices from 4 components [x y z w] to 2 components [x y]   
+                            v0 = [list_polygon_vertex[0][0], list_polygon_vertex[0][1]]
+                            v1 = [list_polygon_vertex[1][0], list_polygon_vertex[1][1]]
+                            v2 = [list_polygon_vertex[2][0], list_polygon_vertex[2][1]]
+
+                            # NOTE: polygon draw with 0 means fill 0 otherwise it means thickness of line (int)
+                            if display_list_type == Engine.DISPLAY_LIST_WIREFRAME:
+                                pg.draw.line(self.screen, color, v0, v1, 2)
+                                pg.draw.line(self.screen, color, v1, v2, 2)
+                                pg.draw.line(self.screen, color, v2, v0, 2)
+                            elif display_list_type == Engine.DISPLAY_LIST_SHADED:
+                                pg.draw.polygon(self.screen, face_color, list_polygon_vertex, 0)  
+                            elif display_list_type == Engine.DISPLAY_LIST_SHADED_OUTLINE:
+                                pg.draw.polygon(self.screen, face_color, list_polygon_vertex, 0)  
+                                pg.draw.line(self.screen, pg.Color('white'), v0, v1, 1)
+                                pg.draw.line(self.screen, pg.Color('white'), v1, v2, 1)
+                                pg.draw.line(self.screen, pg.Color('white'), v2, v0, 1)
+                            else:
+                                SiLog.Unsupported(display_list_type);
 
 
+                        if obj.draw_normals:
+                            self.__draw_normals(list_vertex_world, list_vertex_index, list_normal, face_idx)
 
 
 
@@ -409,7 +389,38 @@ class Engine:
 
 
 
-       
+    ###############################################################################
+    #
+    ###############################################################################
+    def draw_rect_gradient(self, color_top: pg.Color, color_bottom: pg.Color, target_rect: pg.Rect, gradient_direction: 'SiDirection'):
+    
+        # draw colors to a tiny rectangle
+        tiny_rect = pg.Surface((2,2)) 
+
+        if gradient_direction == SiDirection.VERTICAL:
+            pg.draw.line(tiny_rect, color_top, (0,0), (1,0))  
+            pg.draw.line(tiny_rect, color_bottom, (0,1), (1,1))          
+        else:
+            pg.draw.line(tiny_rect, color_top, (0,0), (0,1))  
+            pg.draw.line(tiny_rect, color_bottom, (1,0), (1,1))          
+
+        # scale rectangle up and use smoothing
+        tiny_rect = pg.transform.smoothscale(tiny_rect, (target_rect.width, target_rect.height))
+
+        self.screen.blit(tiny_rect, target_rect) 
+     
+        
+    #dsd move into Si since Si assumes pygame!
+    ###############################################################################
+    #
+    ###############################################################################
+    def draw_text(self, pos:pg.math.Vector2, color:pg.Color, text:str, font:pg.font.Font):    
+
+        text_surface = font.render(text, True, color)
+        text_rect = text_surface.get_rect()
+        text_rect.midtop = (pos.x, pos.y)
+        self.screen.blit(text_surface, text_rect)
+
     ###############################################################################
     #
     ###############################################################################
@@ -420,3 +431,5 @@ class Engine:
         self.clock.tick(self.FPS)
 
 
+
+
