@@ -9,7 +9,7 @@ class LightDirectional:
     ###############################################################################
     #
     ###############################################################################
-    def __init__(self, dir: SiVector3):
+    def __init__(self, dir:pg.math.Vector3):
 
         light_vector = list()
 
