@@ -1,10 +1,16 @@
+import math
+import numpy as np
+from numba import njit
+import pygame as pg
+
+
 from Retro3D.si import *
 
-from . BackgroundInfo import *
 from . ConfigGame import *
 from . ObjFile import *
+from . Model import *
+from . Rez import *
 from . Matrix import *
-from . Mesh import *
 from . Projection import *
 from . Camera import *
 from . Object import *
