@@ -10,23 +10,16 @@ class SiLog:
     #
     #######################################################################################
     @staticmethod
-    def Message(msg: str):
+    def Message(msg):
         print("{}".format(msg))
 
-    ########################################################################################
-    #
-    #######################################################################################
-    @staticmethod
-    def Message(msg: str, val: str):
-        print("{}".format(msg + " [" + str(val) + "]"))
-
 
     ########################################################################################
     #
     #######################################################################################
     @staticmethod
-    def Error(msg: str, val):
-        raise Exception(msg + " [" + str(val) + "]")
+    def Error(err):
+        raise Exception(str(err))
 
     ########################################################################################
     #
@@ -40,6 +33,6 @@ class SiLog:
     #
     #######################################################################################
     @staticmethod
-    def KeyVal(key: str, val: str):
+    def KeyVal(key, val):
         print("{} [{}]".format(key, val))
 
