@@ -14,13 +14,10 @@ class SiRandom:
     def Seed(seed):
 
         #seed 0 means use random based on system time 
-        #python does this by default - so no need to do anyything
+        #python does this by default - so no need to do anything
         if seed == 0:
-            SiLog.KeyVal("random seed", "based on time")  
             return
 
-        SiLog.KeyVal("random seed", seed)   
-
         random.seed(seed)
 
 
@@ -43,3 +40,13 @@ class SiRandom:
     def GetInt(min, max):
         return random.randint(min, max)
 
+    ########################################################################################
+    #
+    #######################################################################################
+    @staticmethod
+    def GetTrueFalse():
+        if SiRandom.GetInt(0,1) == 0:
+            return False
+
+        return True
+
