@@ -22,5 +22,5 @@ class SiTimer:
     def stop(self):
 
         timer_end =  time.perf_counter()
-        SiLog.Message("time", timer_end - self.timer_start)
+        SiLog.KeyVal("time", timer_end - self.timer_start)
 
